package com.winhc.dataworks.flow.touch;

import com.aliyuncs.dataworks_public.model.v20180601.CreateManualDagResponse;
import com.helospark.lightdi.LightDi;
import com.helospark.lightdi.LightDiContext;
import com.helospark.lightdi.annotation.Autowired;
import com.helospark.lightdi.annotation.Service;
import com.winhc.dataworks.flow.touch.bean.*;
import com.winhc.dataworks.flow.touch.configuration.SchemaInit;
import com.winhc.dataworks.flow.touch.service.TouchService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Author: XuJiakai
 * @Date: 2020/6/22 10:45
 * @Description:
 */
@Slf4j
@Service
public class Main {
    @Autowired
    private TouchService touchService;


    public static void main(String[] args) {
        if (args.length != 1) {
            System.out.println("请输入调度的 bizDate");
            System.exit(-9999);
        }
        LightDiContext context = LightDi.initContextByPackage(Main.class.getPackage().getName());
        Main bean = context.getBean(Main.class);
        bean.start(args[0]);
    }

    @SneakyThrows
    private void start(String bizDate) {
        log.info("start");
        List<DataWorksFlowJob> jobs = SchemaInit.LIST;
        List<TaskInfo> collect = jobs.stream()
                .flatMap(dataWorksFlowJob -> {
                    List<DataWorksFlowTask> task = dataWorksFlowJob.getTask();
                    return task
                            .stream()
                            .map(dataWorksFlowTask -> {
                                TaskParam build = TaskParam.builder()
                                        .projectName(dataWorksFlowJob.getProject())
                                        .bizDate(bizDate)
                                        .flowName(dataWorksFlowJob.getFlow())
                                        .nodeParam(dataWorksFlowTask.toNodeParam())
                                        .build();
                                CreateManualDagResponse r = touchService.touch(build);

                                return TaskInfo.builder().project(dataWorksFlowJob.getProject())
                                        .flow(dataWorksFlowJob.getFlow())
                                        .taskName(dataWorksFlowTask.getTaskName())
                                        .dagId(r.getReturnValue()).build();
                            });
                }).collect(Collectors.toList());

        int totalTask = collect.size();

        while (true) {
            int awaitTask = 0;
            int successTask = 0;
            int failedTask = 0;

            int awaitNode = 0;
            int successNode = 0;
            int failedNode = 0;

            boolean flag = true;

            for (TaskInfo taskInfo : collect) {
                Map<String, TaskFlowEnum> query = touchService.query(taskInfo.getProject(), taskInfo.getDagId());
                Map<String, List<String>> status = query.entrySet().stream()
                        .map(e -> {
                            Entry<String, String> entry = new Entry<>();
                            if (e.getValue().equals(TaskFlowEnum.SUCCESS) || e.getValue().equals(TaskFlowEnum.FAILURE)) {
                                entry.setKey(e.getValue().getMsg());
                            } else {
                                entry.setKey("运行中");
                            }
                            return entry;
                        }).collect(Collectors.groupingBy(Entry::getKey, Collectors.mapping(Entry::getValue, Collectors.toList())));
                List<String> success = status.get(TaskFlowEnum.SUCCESS.getMsg());
                List<String> failure = status.get(TaskFlowEnum.FAILURE.getMsg());
                List<String> await = status.get("运行中");

                if (await.size() != 0) {
                    awaitTask++;
                    awaitNode += await.size();
                } else {
                    if (failure.size() != 0) {
                        failedTask++;
                        failedNode += failure.size();
                        log.error("failure node:{}  {}", failedTask, failedNode);
                    } else {
                        successNode += success.size();
                        successTask++;
                    }
                }
                flag = flag && await.size() == 0;
            }
            log.info("\nawait task:{}\ntotal task:{}\nsuccess task:{}\nfailure task:{}\n"
                    , awaitTask
                    , totalTask
                    , successTask
                    , failedTask
            );

            log.info("\nawait node:{}\ntotal node:{}\nsuccess node:{}\nfailure node:{}\n"
                    , awaitNode
                    , awaitNode + successNode + failedNode
                    , successNode
                    , failedNode
            );

            Thread.sleep(10000);

            if (flag) {
                break;
            }
        }
        log.info("end");
    }
}
